@@ -920,7 +920,8 @@ store_result(struct command_state *state, int res)
 {
   int index = get_index(state);
   next_index(state);
-  state->object->store(state->object->arg, (void *) (long) res, index, NULL);
+  state->object->store(state->object->arg, (void *) (ptrdiff_t) res,
+                       index, NULL);
 }
 
 
@@ -970,6 +971,29 @@ parse_delete_reply(struct command_state *state)
 }
 
 
+static
+int
+parse_touch_reply(struct command_state *state)
+{
+  switch (state->match)
+    {
+    case MATCH_TOUCHED:
+      store_result(state, 1);
+      break;
+
+    case MATCH_NOT_FOUND:
+      store_result(state, 0);
+      break;
+
+    default:
+      return MEMCACHED_UNKNOWN;
+    }
+
+  return swallow_eol(state, 0, 1);
+}
+
+
+
 static
 int
 parse_arith_reply(struct command_state *state)
@@ -1119,6 +1143,7 @@ parse_nowait_reply(struct command_state *state)
     case MATCH_EXISTS:
     case MATCH_NOT_FOUND:
     case MATCH_NOT_STORED:
+    case MATCH_TOUCHED:
       return swallow_eol(state, 0, 1);
 
     case MATCH_0: case MATCH_1: case MATCH_2: case MATCH_3: case MATCH_4:
@@ -1422,7 +1447,7 @@ state_prepare(struct command_state *state)
 
       while (count > 0)
         {
-          iov->iov_base = (void *) (buf + (long) (iov->iov_base));
+          iov->iov_base = (void *) (buf + (ptrdiff_t) (iov->iov_base));
           iov += step;
           count -= step;
         }
@@ -1847,7 +1872,7 @@ client_prepare_set(struct client *c, enum set_cmd_e cmd, int key_index,
     size_t str_size =
       sprintf(buf, " " FMT_FLAGS " " FMT_EXPTIME " " FMT_VALUE_SIZE "%s\r\n",
               flags, exptime, value_size, get_noreply(state));
-    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
+    iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1888,7 +1913,7 @@ client_prepare_cas(struct client *c, int key_index,
       sprintf(buf, " " FMT_FLAGS " " FMT_EXPTIME " " FMT_VALUE_SIZE
               " " FMT_CAS "%s\r\n", flags, exptime, value_size, cas,
               get_noreply(state));
-    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
+    iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -1979,7 +2004,7 @@ client_prepare_incr(struct client *c, enum arith_cmd_e cmd, int key_index,
     char *buf = array_end(c->str_buf, char);
     size_t str_size =
       sprintf(buf, " " FMT_ARITH "%s\r\n", arg, get_noreply(state));
-    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
+    iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -2010,7 +2035,39 @@ client_prepare_delete(struct client *c, int key_index,
   {
     char *buf = array_end(c->str_buf, char);
     size_t str_size = sprintf(buf, "%s\r\n", get_noreply(state));
-    iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
+    iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
+    array_append(c->str_buf, str_size);
+  }
+
+  return MEMCACHED_SUCCESS;
+}
+
+
+int
+client_prepare_touch(struct client *c, int key_index,
+                      const char *key, size_t key_len,
+                      exptime_type exptime)
+{
+  static const size_t request_size = 4;
+  static const size_t str_size = sizeof(" " NOREPLY "\r\n");
+
+  struct command_state *state;
+
+  state = get_state(c, key_index, key, key_len, request_size, str_size,
+                    parse_touch_reply);
+  if (! state)
+    return MEMCACHED_FAILURE;
+
+  ++state->key_count;
+
+  iov_push(state, STR_WITH_LEN("touch"));
+  iov_push(state, c->prefix, c->prefix_len);
+  iov_push(state, key, key_len);
+
+  {
+    char *buf = array_end(c->str_buf, char);
+    size_t str_size = sprintf(buf, " " FMT_EXPTIME "%s\r\n", exptime, get_noreply(state));
+    iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
     array_append(c->str_buf, str_size);
   }
 
@@ -2057,7 +2114,7 @@ client_flush_all(struct client *c, delay_type delay,
         size_t str_size =
           sprintf(buf, "flush_all " FMT_DELAY "%s\r\n",
                   (delay_type) (ddelay + 0.5), get_noreply(state));
-        iov_push(state, (void *) (long) array_size(c->str_buf), str_size);
+        iov_push(state, (void *) (ptrdiff_t) array_size(c->str_buf), str_size);
         array_append(c->str_buf, str_size);
       }
     }
